# Tutorials

These tutorials are meant to illustrate certain NAVis functions. As per usual:
have a look at the [API reference](api.md) to find out details about a given
function. There you will also find more examples.


<div class="grid cards" markdown>

-   :material-progress-wrench:{ .lg .middle } __The Basics__

    ---

    These tutorials will teach you the basics of working with NAVis
    neurons:

    - [Basics](source/tutorials/neurons_intro.ipynb)

</div>
<!--
.. grid:: 1 2 3 4

    .. grid-item-card::

        The Basics
        ^^^

        Working with ``navis`` neurons:

        .. toctree::
           :maxdepth: 1

           tutorials/neurons_intro

    .. grid-item-card::

        Import/Export
        ^^^

        Local data:

        .. toctree::
           :maxdepth: 1

           tutorials/local_data_skeletons
           tutorials/local_data_meshes
           tutorials/local_data_voxels
           tutorials/local_data_dotprops
           tutorials/local_data_pickling

        Remote data sources & repositories:

        .. toctree::
           :maxdepth: 1

           tutorials/neuprint
           tutorials/microns_tut
           tutorials/cloudvolume_tut
           tutorials/neuromorpho_tut
           tutorials/insectbraindb

    .. grid-item-card::

        Plotting
        ^^^

        Visualize your neurons

        .. toctree::
           :maxdepth: 1

           tutorials/plotting

    .. grid-item-card::

        Morphology
        ^^^
        Morphometrics:

        .. toctree::
           :maxdepth: 2

           tutorials/morph_analysis

        Processing:

        .. toctree::
           :maxdepth: 2

           tutorials/morph_processing


.. grid:: 1 2 3 4

    .. grid-item-card::

        NBLAST
        ^^^
        Run & interpret NBLAST:

        .. toctree::
           :maxdepth: 2

           tutorials/nblast
           tutorials/nblast_flycircuit
           tutorials/nblast_hemibrain
           tutorials/smat

    .. grid-item-card::

        Interfaces
        ^^^

        Use ``navis`` in combination with other tools:

        .. toctree::
           :maxdepth: 1

           tutorials/neuron
           blender
           tutorials/r_doc

    .. grid-item-card::

        Transformations
        ^^^

        Transform spatial data (e.g. neurons) between brain spaces:

        .. toctree::
           :maxdepth: 2

           tutorials/transforming

    .. grid-item-card::

        Misc
        ^^^
        .. toctree::
           :maxdepth: 2

           tutorials/multiprocessing
           Troubleshooting <troubleshooting> -->


Not seeing what you're looking for? Need additional pointers? Found a broken
example? Open an [Issue](https://github.com/navis-org/navis/issues) or
use [Discussions](https://github.com/navis-org/navis/discussions) on Github
to ask questions!
